#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>

// User config
const char* wifi_ssid = "SYSDOCU";              // SSID 이름 입력
const char* wifi_password = "12345678";         // Wi-Fi 패스워드 입력
const char* rack_no = "2312";                   // 기타 정보 (필요시)
const unsigned int writeInterval = 5000;        // 반복 시간 (1000 = 1초). 사실상 3초 정도의 시간이 더 소요된다.
const char* https_host = "sysdocu.tistory.com"; // 보낼 서버
const int https_port = 443;                     // https 포트

// DHT config
#define DHTPIN 2
#define DHTTYPE DHT11
DHT_Unified dht(DHTPIN, DHTTYPE);
uint32_t delayMS;
int status = WL_IDLE_STATUS;
float myTemperature = 0, myHumidity = 0; 

WiFiClientSecure client;
void setup() {
  Serial.begin(115200);

  // 고정IP 를 사용하려는 경우 주석 제거
/*
  IPAddress ip(11, 22, 33, 44);
  IPAddress dns(8, 8, 8, 8);
  IPAddress gateway(11, 22, 33, 1);
  IPAddress subnet(255, 255, 255, 0);
  WiFi.config(ip, dns, gateway, subnet);
*/  
  WiFi.begin(wifi_ssid, wifi_password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  
  // Hidden ESP8266 SoftAP (Default : show)
  WiFi.softAP("ESP", "ThisIsHiddenAp", 1, 1);

  // Initialize device.
  dht.begin();
  
  sensor_t sensor;
  dht.temperature().getSensor(&sensor);
  dht.humidity().getSensor(&sensor);
  
  // Set delay between sensor readings based on sensor details.
  delayMS = sensor.min_delay / 1000;

  // Set Client to insecure
  client.setInsecure();
}

void loop() {
  // Read data from DHT
  // Delay between measurements.
  delay(delayMS);

  // Get temperature event and print its value.
  sensors_event_t event;  
  dht.temperature().getEvent(&event);
  if (isnan(event.temperature)) {
    Serial.println("Error reading temperature!");
  }
  else {
    // Update temperature and humidity
    myTemperature = (float)event.temperature;
    Serial.print("Temperature: ");
    Serial.print(myTemperature);
    Serial.println(" C");
  }
  
  // Get humidity event and print its value.
  dht.humidity().getEvent(&event);
  if (isnan(event.relative_humidity)) {
    Serial.println("Error reading humidity!");
  }
  else {
     myHumidity = (float)event.relative_humidity;
    Serial.print("Humidity: ");
    Serial.print(myHumidity);
    Serial.println("%");
  }
  
  // Use WiFiClientSecure class to create TLS connection
  Serial.print("********** connecting to HOST : ");
  Serial.println(https_host);
  if (!client.connect(https_host, https_port)) {
    Serial.println("-> connection failed");
    //return;
  }

  // Create a URL for the request
  String url = "/insert.html?rack=";
  url += rack_no;
  url += "&temp=";
  url += myTemperature;
  url += "&humi=";
  url += myHumidity;
  
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + https_host + "\r\n" +
               "Connection: close\r\n\r\n");

  while (client.connected()) {
    String line = client.readStringUntil('\n');
    if (line == "\r") {
        String line = client.readStringUntil('\n');
        Serial.println("********** replay:");
        Serial.println(line);
        Serial.println("********** closing connection");
      
        break;
    }
  }

  delay(writeInterval); // delay in ms
}
